/****************************************************************************
    
    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#include "selection_widget.h"
#include "newt_parent_widget.h"
#include "config.h"

template <class T>
SelectionWidget<T>::SelectionWidget(Glib::ustring label, NewtParentWidget* parent, ControlEnum ctrlID)
 : m_refCBTreeModel(Gtk::ListStore::create(m_cbColumns))
 , m_label(label)
 , m_ctrlID(ctrlID)
{
    m_comboBox.set_model(m_refCBTreeModel);
    m_comboBox.signal_changed().connect(sigc::mem_fun(*this, &SelectionWidget<T>::on_signal_changed));
    pack_start(m_label, false, false);
    pack_end(m_comboBox, true, true);

    m_parent = parent;
    if (m_parent)
    {
        m_parent->signal_value_should_change().connect(sigc::mem_fun(*this, &SelectionWidget<T>::on_value_should_change));
    }
}

template <class T>
void SelectionWidget<T>::on_value_should_change(ControlEnum ctrl, stk::StkFloat val)
{
    if (ctrl == m_ctrlID)
        SetActive((T)val);
}

template <class T>
T SelectionWidget<T>::GetActive()
{
    Gtk::TreeIter active = m_comboBox.get_active();
    return (*active)[m_cbColumns.m_id];
}

template <class T>
void SelectionWidget<T>::SetActive(T id)
{
    Glib::ustring path = Glib::ustring::compose("%1", (int)id);
    //TRACE1("HERE", path.c_str());
    Gtk::TreeModel::iterator it = m_refCBTreeModel->get_iter(path);

    m_comboBox.set_active(it);
}

template <class T>
void SelectionWidget<T>::AddItems(T ids[], const char* names[], int num)
{
    for (int i = 0; i < num; i++)
    {
        AddItem(ids[i], names[i]);
    }

    m_comboBox.pack_start(m_cbColumns.m_name);
}

template <class T>
void SelectionWidget<T>::AddItem(T id, Glib::ustring name)
{
    Gtk::TreeModel::Row row;
    Gtk::TreeModel::Path path;

    row = *(m_refCBTreeModel->append());
    row[m_cbColumns.m_id] = id;
    row[m_cbColumns.m_name] = name;
    //path = m_refCBTreeModel->get_path(row);

    //TRACE1("Added to Combobox", path.to_string().c_str());
}

template <class T>
void SelectionWidget<T>::on_signal_changed()
{
    T val = GetActive();
    if (m_parent)
        m_parent->EmitValueChange(m_ctrlID, (stk::StkFloat)val);

    m_signalChanged.emit();
}
